
from ..utils.log_manage import base_logger
from tqdm import tqdm

def filter_data(data:list[dict], filter_set:set, main_key_column:str)-> list[dict]:
    """将data中的main_key_columns字段根据filter_set的数据进行过滤

    Args:
        data (list): 待过滤的数据
        filter_set (_type_): 需要过滤的数据
        main_key_column (_type_): 待过滤数据的key

    Returns:
        list[dict]: 过滤后的数据
    """
    new_data = []
    for item in data:
        if main_key_column not in item:
            raise RuntimeError(f"data中没有字段{main_key_column=}")
        sub_item = item[main_key_column]
        if sub_item in filter_set:
            continue
        new_data.append(item)
    base_logger.info(f"原始数据大小:{len(data)}, 过滤后大小:{len(new_data)}")
    return new_data


def drop_duplicates(data: list[dict], main_key_column:str)-> list[dict]:
    """去除data中 main_key_columns字段重复的数据

    Args:
        data (list): 由dict存储的数据
        main_key_column (str): 需要去重的key

    Returns:
        list[dict]: 去重后的数据
    """
    temp_set = set()
    new_data = []
    for item in tqdm(data, desc="去重中"):
        if main_key_column not in item:
            base_logger.warning(f"不存在对应的key:{main_key_column=}\n{item=}\n已跳过")
            continue
        else:
            key = item[main_key_column]
            if key not in temp_set:
                new_data.append(item)
                temp_set.add(key)
    return new_data


def remove_columns(data: list[dict], key_list: list|str)-> list[dict]:
    """删除data中对应key_list对应的数据

    Args:
        data (list): 由dict存储的数据
        key_list (list|str): 需要删除的key

    Returns:
        list[dict]: 删除后的数据
    """
    
    if isinstance(key_list, str):
        key_list = [key_list]
    
    new_data = []
    for item in tqdm(data, desc="删除对应列中"):
        for k in key_list:
            if k in item:
                del item[k]
        
        new_data.append(item)
    return new_data


"""
The caoyizhen_basetool library provides a tool to help you to dealing with data in Python.
"""

from .file import read_file, save_file, add_suffix_file, return_to_jsonl
from .process import BaseMultiThreading, filter_data, remove_columns, drop_duplicates
from .utils import singleton, LoggerManager, base_logger


# -*- encoding: utf-8 -*-
# @Time    :   2025/10/11 22:16:26
# @File    :   MultiThreadProcess.py
# @Author  :   ciaoyizhen
# @Contact :   yizhen.ciao@gmail.com
# @Function:   多线程的消费者生产者进程处理
import json
from pathlib import Path
from concurrent.futures import as_completed, ThreadPoolExecutor
from ..utils import base_logger
from ..file import save_file, read_file
from tqdm import tqdm



class BaseMultiThreading():
    """
    基类, 实现多线程的消费者生产者的处理, 实现边处理边存储
    """
    def __init__(self, max_workers:int, save_path: str|Path=None, *, file_type:str|Path=None, **kwargs):
        """_summary_

        Args:
            max_workers (int): 并发数
            single_file_size (int): 临时存储时，单个文件的大小
            save_path (str|Path): 最终完整保存的文件
            file_type (str|Path): 文件存储类型
        """
        self.max_workers = max_workers
        self.save_path = Path(save_path)
        self.file_type = file_type
        
        if self.file_type is None:
            self.file_type = self.save_path.suffix.lstrip(".")
            
        if self.file_type not in {"json", "jsonl", "xlsx", "csv"}:
            raise RuntimeError(f"传入的file_type不符合要求或你的文件后缀不符合要求")

        self.post_init(**kwargs)
    
    def post_init(self, **kwargs):
        pass
    
    def single_data_process(self, item:dict)->dict:
        """
        这个函数实现单个数据怎么处理，输入是一个数据，进行处理，返回一个数据
        需要用户自定义实现
        """
        raise NotImplementedError(f"未实现函数 single_data_process, 该函数需要解决每个数据要怎么")
            
    
    def __call__(self, data:list):
        with ThreadPoolExecutor(max_workers=self.max_workers, thread_name_prefix="线程处理数据") as exec, \
            tqdm(total=len(data), desc=f"{self.max_workers}并发处理中") as p_bar, \
            open(self.save_path, "w", encoding="utf-8") as f:
            try:
                futures_list = []
                for item in data:
                    future = exec.submit(self.single_data_process, item)
                    future.add_done_callback(lambda x: p_bar.update(1))
                    futures_list.append(future)
                
                for future in as_completed(futures_list):
                    result = future.result()
                    result = json.dumps(result, ensure_ascii=False)
                    f.write(result + "\n")
                    f.flush()
            except KeyboardInterrupt:
                exit()
                exec.shutdown(cancel_futures=True)
            except Exception:
                import traceback
                base_logger.error(traceback.format_exc())
                


# -*- encoding: utf-8 -*-
# @Time    :   2025/10/12 12:48:14
# @File    :   utils.py
# @Author  :   ciaoyizhen
# @Contact :   yizhen.ciao@gmail.com
# @Function:   通用工具
from functools import wraps
from threading import Lock

def singleton(cls):
    """线程安全的单例模式装饰器"""
    instances = {}
    lock = Lock()

    @wraps(cls)
    def get_instance(*args, **kwargs):
        # 双重检查锁，防止多线程竞争
        if cls not in instances:
            with lock:
                if cls not in instances:
                    instances[cls] = cls(*args, **kwargs)
        return instances[cls]

    return get_instance


